#pragma	once

#include "types.h"
#include "image.h"

class	cGUIControl
{
private:

	bool			mActive;	//     (..    )
	bool			mVisible;	//     (..   )
	bool			mFocus;		//     (..   )
	bool			mPress;		//       
	cGUIControl		*mNext;

public:

	rect		mBounds;	//    ()
	color		mBColor;	//   
	image		*mBImg;		//   ( )

	cGUIControl(){ SetDefault(); }
	cGUIControl(float px,float py,float sx,float sy) : mBounds(px,py,sx,sy){ SetDefault(); }

	bool IsActive() const { return mActive; }
	bool IsVisible() const { return mVisible; }
	bool IsCaptured() const { return mFocus; }
	bool IsPressed() const { return mPress; }

	// setters / getters
	void SetDefault()
	{
		mActive = true;
		mVisible = true;
		mFocus = false;
		mPress = false;
		mBColor.Set(128,128,128);
		mBImg = 0;
		mNext = 0;
	}
	cGUIControl& GetNext() const { return *mNext; }

	void Press(){ mPress = true; OnPress(); }
	void UnPress(){ mPress = false; OnUnPress(); }
	void Enable() { mActive = true; OnEnable(); }
	void Disable(){ mActive = false; OnDisable(); }
	void Show() { mVisible = true; OnShow(); }
	void Hide(){ mVisible = false; OnHide(); }
	void Capture() { mFocus = true; OnFocusGain(); }
	void Release(){ mFocus = false; OnFocusLost(); }
	void Scroll(int pos){ if(mFocus) OnScroll(pos); }

	void Add(cGUIControl& c){ if(!mNext) mNext = &c; else mNext->Add(c); }

	virtual void OnDraw(){} //   -
	virtual void OnUpdate(float delta_time){} //     (       ..)

protected:

	virtual void OnPress(){}         //  -,   
	virtual void OnUnPress(){}     //  -,   
	virtual void OnEnable(){}       //  -,    
	virtual void OnDisable(){}      //  -,     
	virtual void OnShow(){}         //  -,    
	virtual void OnHide(){}          //  -,    
	virtual void OnFocusGain(){}       //  -,     ()
	virtual void OnFocusLost(){}       //  -,     ( )
	virtual void OnScroll(int pos){}  //  -,      (  ScrollBar)
};
class	cGUIButton : public cGUIControl
{
private:
public:

	string		mCaption;	//  
	color		mTColor;	//    
//	font		*mFont;		// 

	cGUIButton(string caption = 0) : cGUIControl(),mCaption(caption) { SetDefault(); }
	cGUIButton(float px,float py,float sx,float sy,string caption = 0) : cGUIControl(px,py,sx,sy),mCaption(caption){ SetDefault(); }

	//	setters / getters
	void SetDefault()
	{
		mTColor.Set(128,128,128);
	//		mFont = 0;
	}

	virtual void OnDraw();
};
class	cGUIManager
{
private:

	uint			mCCount;
	cGUIControl		*mControls;
	cGUIControl		*mLast;

public:

	cGUIManager() : mCCount(0),mControls(0),mLast(0){}

	void Add(cGUIControl& c);
	void ProcFocus(vec2& cursor);
	void ProcPress(bool state,vec2& cursor);
	void Draw();
	void Update(float time_delta);
};