#ifndef		WIN32_LEAN_AND_MEAN
#define		WIN32_LEAN_AND_MEAN
#endif

#include <windows.h>
#include <gl//gl.h>
#include <gl//glu.h>
#include "image.h"

bool	image::Load(FILE *fp)
{
	if(!fp)	return	false;

	BITMAPFILEHEADER	fh;
	BITMAPINFOHEADER	ih;

	if(fread(&fh,1,sizeof(BITMAPFILEHEADER),fp) != sizeof(BITMAPFILEHEADER))
		return	false;

	if(fh.bfType != 'MB')	return	false;

	if(fread(&ih,1,sizeof(BITMAPINFOHEADER),fp) != sizeof(BITMAPINFOHEADER))
		return	false;

	Free();

	mSX = ih.biWidth;
	mSY = ih.biHeight;

	switch(ih.biBitCount)
	{
	case 24:
		{
			mNCC = GL_RGB;
			mFormat = GL_BGR_EXT;
		}
		break;

	case 32:
		{
			mNCC = GL_RGBA;
			mFormat = GL_BGRA_EXT;
		}
		break;

	default:
		return	false;
	}

	uint	sizerow = mSX * (ih.biBitCount / 8);
	uint	stride = ((4 - (sizerow % 2)) % 2);
	uint	size = sizerow * mSY;

	mData = new byte[size];
	if(!mData)	return	false;

	fseek(fp,fh.bfOffBits,SEEK_SET);

	if(stride == 0)
	{
		if(fread(mData,1,size,fp) != size)	return	false;
	}
	else
	{
		byte	*data = mData;
		for(int i=0;i<mSX;i++)
		{
			if(fread(data,1,sizerow,fp) != sizerow)	return	false;
			fseek(fp,stride,SEEK_CUR);
			data = (data + sizerow);
		}
	}

	SetToMemory();

	return	true;
}
bool	image::Load(const string fname)
{
	FILE	*fp = fopen(fname,"rb");
	if(!fp)	return	false;

	return	Load(fp);
}
bool	image::Load(uint resource)
{
	HBITMAP		bmp;
	SIZE		size;

	bmp = LoadBitmap(GetModuleHandle(0),MAKEINTRESOURCE(resource));
	if(!bmp)	return	false;

	GetBitmapDimensionEx(bmp,&size);

	Free();

	mSX = size.cx;
	mSY = size.cy;

	mData = new byte[mSX * mSY * 3];
	if(!mData)	return	false;

	GetBitmapBits(bmp,(mSX * mSY * 3),mData);
	SetToMemory();

	return	true;
}
void	image::SetToMemory()
{
	if(mTexID == 0)	glGenTextures(1,&mTexID);
	glBindTexture(GL_TEXTURE_2D,mTexID);
	gluBuild2DMipmaps(GL_TEXTURE_2D,mNCC,mSX,mSY,mFormat,GL_UNSIGNED_BYTE,mData);
}
void	image::Bind()
{
	glBindTexture(GL_TEXTURE_2D,mTexID);
}
void	image::Free()
{
	if(mTexID != 0)
	{
		glDeleteTextures(1,&mTexID);
		mTexID = 0;
	}
	if(mData)
	{
		delete[] mData;
		mData = 0;
	}
}